<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:util="http://www.pnp-software.com/util"
  xmlns:xd="http://www.pnp-software.com/XSLTdoc"
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  version="2.0" exclude-result-prefixes="#all">
  
  <xd:doc type="stylesheet">
    <xd:short>This stylesheet converts an XFeature project (normally named *.xfp) 
    file into an ant files that the aplication generation and display
    model generation</xd:short>
    <xd:detail>
      Several ant targets are created.
    </xd:detail>
    <xd:cvsId>$Id: xfproject2ant.xsl,v 1.1 2005/11/10 10:25:05 rohliko Exp $</xd:cvsId>
    <xd:author>rohlik</xd:author>
    <xd:copyright>2005, ETH and P&amp;P Software GmbH</xd:copyright>
  </xd:doc>
  
  <xsl:output indent="yes" method="xml"/>
  
  <xd:doc>
    Main template. Apply templates for XFeature project element
  </xd:doc>
  <xsl:template match="/XFeatureProject">
    <project default="default" name="{@name}">
      <description>Ant Build file for the XFeature</description>

      <!--<property name="GeneralFiles" location="../general_files" />-->

      <property name="MetaModel" location="{FamilyXFMM/@location}"/>
      <property name="DisplayModel" location="{FamilyXDM/@location}" />
      <property name="TmpLoc" location="{TmpLoc/@location}" />
      <property name="FamilyXFM" location="{FamilyXFM/@location}" />
      <property name="FamilyXFMM" location="{FamilyXFMM/@location}" />
      <property name="FamilyXDM" location="{FamilyXDM/@location}" />
      <property name="AppXFMM" location="{AppXFMM/@location}" />
      <property name="AppXDM" location="{AppXDM/@location}" />
      
      <target name="validate">
        <echo>Test the configuration</echo>
        <!--<ant target="ValidateFeatureMetaModel" antfile="build.xml" dir="${{GeneralFiles}}" inheritall="false">-->
        <ant target="ValidateFeatureMetaModel" antfile="build.xml" dir="." inheritall="false">
          <property name="MetaModel" location="${{MetaModel}}" />
          <property name="DisplayModel" location="${{DisplayModel}}" />
          <property name="TmpLoc" location="${{TmpLoc}}"/>
        </ant>
      </target>
      
      <target name="generate">
        <ant target="clean"/>
        <fail unless="FamilyXFM" message="The XSD Application Generator: Family model filename has to specified." />
        <fail unless="FamilyXDM" message="The XSD Application Generator: Family display model filename has to specified." />
        <fail unless="AppXFMM" message="The XSD Application Generator: Filename of the application meta-model has to specified." />
        <fail unless="AppXDM" message="The XSD Application Generator: Filename of the application display model has to specified." />
        <fail unless="TmpLoc" message="The XSD Application Generator: Temp directory not specified." />
        <ant target="GenerateApplicationSchema"/>
        <ant target="GenerateAndCheckDisplayModel"/>
        <ant target="clean"/>
      </target>
         
      <target name="GenerateApplicationSchema" description="Generates the application metamodel (*.xfmm) from family model (*.xfm).">
        <xmlvalidate file="${{FamilyXFM}}" lenient="false" failonerror="true" warn="true">
          <attribute name="http://xml.org/sax/features/namespaces" value="true"/>
          <attribute name="http://apache.org/xml/features/validation/schema" value="true"/>
        </xmlvalidate>
        <xsl:variable name="AppMetaModelGenFileCount" select="count(AppMetaModelGen/File)"/>
        
        <!-- The following just doesn't work -->
        <!--
        <xsl:variable name="AppMetaModelGenFiles">
          OK: <xsl:value-of select="$Orders/@location"/>
          <xsl:perform-sort select="AppMetaModelGen/File">
            <xsl:sort select="@order"/>
          </xsl:perform-sort>
        </xsl:variable>
        <xsl:comment> AppMetaModelGenFiles: <xsl:value-of select="$AppMetaModelGenFiles/@location"/> (<xsl:value-of select="count($AppMetaModelGenFiles)"/>)</xsl:comment>
        -->
        
        <!--<xsl:comment>Number of files that forms the generator is <xsl:value-of select="$AppMetaModelGenFileCount"/></xsl:comment>-->
        <xsl:choose>
          <xsl:when test="$AppMetaModelGenFileCount = 1">
            <xsl:comment>Generator is in one file only</xsl:comment>
            <xslt force="true" style="{AppMetaModelGen/File/@location}" in="${{FamilyXFM}}" out="${{AppXFMM}}">
              <factory name="net.sf.saxon.TransformerFactoryImpl"/>
            </xslt>
          </xsl:when>
          <xsl:otherwise>
            <xsl:variable name="GenFiles" select="AppMetaModelGen/File"/>
            
            <!-- if size = first -->
            <!--<xsl:comment> if size = first </xsl:comment>-->
            <xslt force="true" style="{$GenFiles[1]/@location}" in="${{FamilyXFM}}" out="${{TmpLoc}}/AppGenTmp(1).xml">
              <factory name="net.sf.saxon.TransformerFactoryImpl"/>
            </xslt>
            
            <!-- if size = middle -->
            <!--<xsl:comment> if size = middle </xsl:comment>-->
            <xsl:for-each select="AppMetaModelGen/File[position()>1 and position() lt $AppMetaModelGenFileCount]"> 
              <xsl:sort select="./@order" data-type="number"/>
              <xsl:comment> <xsl:value-of select="./@location"/> </xsl:comment>
              <xslt force="true" style="{@location}" in="${{TmpLoc}}/AppGenTmp({@order -1}).xml" out="${{TmpLoc}}/AppGenTmp({@order}).xml">
                <factory name="net.sf.saxon.TransformerFactoryImpl"/>
              </xslt>
            </xsl:for-each>
            
            <!-- if size = last -->
            <!--<xsl:comment> if size = last </xsl:comment>-->
            <xslt force="true" style="{$GenFiles[last()]/@location}" in="${{TmpLoc}}/AppGenTmp({$AppMetaModelGenFileCount - 1}).xml" out="${{AppXFMM}}">
              <factory name="net.sf.saxon.TransformerFactoryImpl"/>
            </xslt>
            
            <!--<xsl:comment> end of "otherwise" </xsl:comment>-->
          </xsl:otherwise>
        </xsl:choose>
      </target>
      
      <target name="GenerateAndCheckDisplayModel" description="Generates the application display model (*.xdm) from family display model (*.xdm) and family meta-model (*.xfmm). Then it checks that the generated display model is complete.">
        <!--
        <xslt force="true" style="{AppDisplayModelGen/File/@location}" in="${{FamilyXDM}}" out="${{TmpLoc}}/xfmm2xdm.xsl">
          <factory name="net.sf.saxon.TransformerFactoryImpl"/>
        </xslt>
        -->
        <xsl:variable name="AppDisplayModelGenFileCount" select="count(AppDisplayModelGen/File)"/>
        <!--<xsl:comment>Number of files that forms the generator is <xsl:value-of select="$AppMetaModelGenFileCount"/></xsl:comment>-->
        <xsl:choose>
          <xsl:when test="$AppDisplayModelGenFileCount = 1">
            <xsl:comment>Generator is in one file only</xsl:comment>
            <xslt force="true" style="{AppDisplayModelGen/File/@location}" in="${{FamilyXDM}}" out="${{TmpLoc}}/xfmm2xdm.xsl">
              <factory name="net.sf.saxon.TransformerFactoryImpl"/>
            </xslt>
          </xsl:when>
          <xsl:otherwise>
            <xsl:variable name="GenFiles" select="AppDisplayModelGen/File"/>
            
            <!-- if size = first -->
            <!--<xsl:comment> if size = first </xsl:comment>-->
            <xslt force="true" style="{$GenFiles[1]/@location}" in="${{FamilyXDM}}" out="${{TmpLoc}}/AppGenTmp(1).xml">
              <factory name="net.sf.saxon.TransformerFactoryImpl"/>
            </xslt>
            
            <!-- if size = middle -->
            <!--<xsl:comment> if size = middle </xsl:comment>-->
            <xsl:for-each select="AppDisplayModelGen/File[position()>1 and position() lt $AppDisplayModelGenFileCount]"> 
              <xsl:sort select="./@order" data-type="number"/>
              <xsl:comment> <xsl:value-of select="./@location"/> </xsl:comment>
              <xslt force="true" style="{@location}" in="${{TmpLoc}}/AppGenTmp({@order -1}).xml" out="${{TmpLoc}}/AppGenTmp({@order}).xml">
                <factory name="net.sf.saxon.TransformerFactoryImpl"/>
              </xslt>
            </xsl:for-each>
            
            <!-- if size = last -->
            <!--<xsl:comment> if size = last </xsl:comment>-->
            <xslt force="true" style="{$GenFiles[last()]/@location}" in="${{TmpLoc}}/AppGenTmp({$AppDisplayModelGenFileCount - 1}).xml" out="${{TmpLoc}}/xfmm2xdm.xsl">
              <factory name="net.sf.saxon.TransformerFactoryImpl"/>
            </xslt>
            
            <!--<xsl:comment> end of "otherwise" </xsl:comment>-->
          </xsl:otherwise>
        </xsl:choose>
        
        <xslt force="true" style="${{TmpLoc}}/xfmm2xdm.xsl" in="${{AppXFMM}}" out="${{AppXDM}}">
          <factory name="net.sf.saxon.TransformerFactoryImpl"/>
        </xslt>			
        <!--<xslt force="true" style="${{GeneralFiles}}/DisplayModelCompiler.xsl" in="${{AppXFMM}}" out="${{TmpLoc}}/DisplayModelVerifier.xsl">-->
        <xslt force="true" style="DisplayModelCompiler.xsl" in="${{AppXFMM}}" out="${{TmpLoc}}/DisplayModelVerifier.xsl">
            <factory name="net.sf.saxon.TransformerFactoryImpl"/>
        </xslt>			
        <xslt force="true" style="${{TmpLoc}}/DisplayModelVerifier.xsl" in="${{AppXDM}}" out="${{TmpLoc}}/DisplayModelVerificationResults.txt">
          <factory name="net.sf.saxon.TransformerFactoryImpl"/>
        </xslt>			
        <echo message="MESSAGE: The verification results are written to file 'DisplayModelVerificationResults.txt'"/>
      </target>
      
      <target name="clean">
        <delete file="${{TmpLoc}}/DisplayModelVerifier.xsl"/>
        <delete file="${{TmpLoc}}/xfmAddUniqueNames.xfm"/>
        <delete file="${{TmpLoc}}/xfmm2xdm.xsl"/>
        <delete file="${{TmpLoc}}/xfmMacroProcessing.xfm"/>
      </target>	
      
    </project>
  </xsl:template>
</xsl:stylesheet>  
